﻿using C1.Util.DX.Direct2D;
using C1.WPF.Core;
using C1.WPF.Document;
using C1.WPF.ImportServices.ReportingServiceWPF;
using C1.WPF.Input;
using C1.WPF.Report;
using C1.WPF.Viewer;
using System.Diagnostics;
using System.Printing;
using System.Resources;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Navigation;

namespace ReportPrintDemo
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        readonly List<C1ComboBox> _pageCombos = [];
        readonly List<C1TextBox> _marginBoxes = [];

        public MainWindow()
        {
            InitializeComponent();

            var report = new FlexReport();
            report.Load(@"..\..\..\SampleReport.flxr", "Sample Report");
            flexViewer1.DocumentSource = report;

            flexViewer1.Loaded += FlexViewer1_Loaded;
        }

        private void FlexViewer1_Loaded(object sender, RoutedEventArgs e)
        {
            var frame = flexViewer1.Template.FindName("toolFrame", flexViewer1) as Frame;
            if (frame is not null) frame.LoadCompleted += Frame_LoadCompleted;
        }

        private void Frame_LoadCompleted(object sender, NavigationEventArgs e)
        {
            _ = Dispatcher.BeginInvoke(System.Windows.Threading.DispatcherPriority.Loaded, () =>
            {
                var frame = sender as Frame;
                var contentPresenter = frame?.Template.FindName("PART_FrameCP", frame) as ContentPresenter;

                if (contentPresenter is not null && contentPresenter.Content is not null)
                {
                    var psv = contentPresenter.Content as PageSettingsView;
                    var pageSettingsChildren = ((((psv.Content as Grid).Children[1] as Grid).Children[0] as ScrollViewer).Content as StackPanel).Children;

                    #region "Apply Settings" button

                    var btn = ((psv.Content as Grid).Children[1] as Grid).Children[1] as C1Button;
                    EventManager.RemoveClickEvent(btn);
                    btn.Click += Btn_Click;

                    #endregion

                    #region "Orientation" panel

                    var stack = pageSettingsChildren[0] as StackPanel;
                    var combo = stack.Children[2] as C1ComboBox;
                    if (!_pageCombos.Contains(combo)) _pageCombos.Add(combo);

                    #endregion

                    #region "Size" panel

                    stack = pageSettingsChildren[1] as StackPanel;
                    combo = stack.Children[2] as C1ComboBox;
                    if (!_pageCombos.Contains(combo)) _pageCombos.Add(combo);

                    #endregion

                    if (flexViewer1.DocumentSource is not FlexReport report) return;

                    #region "Top Margin" panel

                    stack = pageSettingsChildren[2] as StackPanel;
                    var label = stack.Children[0] as TextBlock;
                    label.Text = "Top Margin (in mm)";
                    var text = stack.Children[2] as C1TextBox;
                    EventManager.RemoveTextChanged(text);
                    text.Tag = stack.Children[1] as TextBlock;
                    text.TextChanged += Text_TextChanged;
                    //if (double.TryParse(text.Text, out double valueInInches))
                    //{
                    var twips = new Unit(report.Layout.MarginTop, UnitTypeEnum.Twip);
                    text.Text = Math.Round(twips.Convert(UnitTypeEnum.Mm), 2).ToString();
                    if (!_marginBoxes.Contains(text)) _marginBoxes.Add(text);
                    //}

                    #endregion

                    #region "Bottom Margin" panel

                    stack = pageSettingsChildren[3] as StackPanel;
                    label = stack.Children[0] as TextBlock;
                    label.Text = "Bottom Margin (in mm)";
                    text = stack.Children[2] as C1TextBox;
                    EventManager.RemoveTextChanged(text);
                    text.Tag = stack.Children[1] as TextBlock;
                    text.TextChanged += Text_TextChanged;
                    //if (double.TryParse(text.Text, out valueInInches))
                    //{
                    twips = new Unit(report.Layout.MarginBottom, UnitTypeEnum.Twip);
                    text.Text = Math.Round(twips.Convert(UnitTypeEnum.Mm), 2).ToString();
                    if (!_marginBoxes.Contains(text)) _marginBoxes.Add(text);
                    //}
                    #endregion

                    #region "Left Margin" panel

                    stack = pageSettingsChildren[4] as StackPanel;
                    label = stack.Children[0] as TextBlock;
                    label.Text = "Left Margin (in mm)";
                    text = stack.Children[2] as C1TextBox;
                    EventManager.RemoveTextChanged(text);
                    text.Tag = stack.Children[1] as TextBlock;
                    text.TextChanged += Text_TextChanged;
                    //if (double.TryParse(text.Text, out valueInInches))
                    //{
                    twips = new Unit(report.Layout.MarginLeft, UnitTypeEnum.Twip);
                    text.Text = Math.Round(twips.Convert(UnitTypeEnum.Mm), 2).ToString();
                    if (!_marginBoxes.Contains(text)) _marginBoxes.Add(text);
                    //}

                    #endregion

                    #region "Right Margin" panel

                    stack = pageSettingsChildren[5] as StackPanel;
                    label = stack.Children[0] as TextBlock;
                    label.Text = "Right Margin (in mm)";
                    text = stack.Children[2] as C1TextBox;
                    EventManager.RemoveTextChanged(text);
                    text.Tag = stack.Children[1] as TextBlock;
                    text.TextChanged += Text_TextChanged;
                    //if (double.TryParse(text.Text, out valueInInches))
                    //{
                    twips = new Unit(report.Layout.MarginRight, UnitTypeEnum.Twip);
                    text.Text = Math.Round(twips.Convert(UnitTypeEnum.Mm), 2).ToString();
                    if (!_marginBoxes.Contains(text)) _marginBoxes.Add(text);
                    //}

                    #endregion
                }
            });
        }

        #region "Wrong Input" handling using TextChanged event
        private void Text_TextChanged(object sender, TextChangedEventArgs e)
        {
            if (sender is C1TextBox textbox)
            {
                var errorBlock = textbox.Tag as TextBlock;

                if (double.TryParse(textbox.Text, out double mm))
                {
                    if (mm < 0 || mm > 381)
                    {
                        errorBlock.Text = "Value must be between 0 and 381";
                        errorBlock.Visibility = Visibility.Visible;
                    }
                    else
                    {
                        errorBlock.Text = string.Empty;
                        errorBlock.Visibility = Visibility.Collapsed;
                    }
                }
                else
                {
                    if (string.IsNullOrEmpty(textbox.Text) || string.IsNullOrWhiteSpace(textbox.Text))
                    {
                        errorBlock.Text = "An empty string is not a valid value";
                        errorBlock.Visibility = Visibility.Visible;
                    }
                    else
                    {
                        errorBlock.Text = $"Can't convert '{textbox.Text}' to a numeric value";
                        errorBlock.Visibility = Visibility.Visible;
                    }
                }
            }
        }

        #endregion

        private void Btn_Click(object sender, RoutedEventArgs e)
        {
            if (flexViewer1.DocumentSource is not FlexReport report) return;

            #region "Orientation" Setting

            var orientationCombo = _pageCombos[0];
            if (Enum.TryParse(orientationCombo.SelectedValue.ToString(), out OrientationEnum orientation))
            {
                report.Layout.Orientation = orientation;
            }
            #endregion

            #region "Page Size" Setting

            var sizeCombo = _pageCombos[1];
            PageMediaSizeName size = _sizes[sizeCombo.SelectedIndex];
            report.Layout.PaperSize = size;

            #endregion

            #region "Margin" Setting

            if (double.TryParse(_marginBoxes[0].Text, out double result))
            {
                result = Math.Round(result, 2);
                var mm = new Unit(result, UnitTypeEnum.Mm);
                report.Layout.MarginTop = mm.Convert(UnitTypeEnum.Twip);
            }
            if (double.TryParse(_marginBoxes[1].Text, out result))
            {
                result = Math.Round(result, 2);
                var mm = new Unit(result, UnitTypeEnum.Mm);
                report.Layout.MarginBottom = mm.Convert(UnitTypeEnum.Twip);
            }
            if (double.TryParse(_marginBoxes[2].Text, out result))
            {
                result = Math.Round(result, 2);
                var mm = new Unit(result, UnitTypeEnum.Mm);
                report.Layout.MarginLeft = mm.Convert(UnitTypeEnum.Twip);
            }
            if (double.TryParse(_marginBoxes[3].Text, out result))
            {
                result = Math.Round(result, 2);
                var mm = new Unit(result, UnitTypeEnum.Mm);
                report.Layout.MarginRight = mm.Convert(UnitTypeEnum.Twip);
            }

            #endregion

            report.Render();
        }

        static readonly List<PageMediaSizeName> _sizes = new List<PageMediaSizeName>(70)
        {
            PageMediaSizeName.Unknown,
            PageMediaSizeName.NorthAmerica10x11,
            PageMediaSizeName.NorthAmerica10x14,
            PageMediaSizeName.NorthAmerica11x17,
            PageMediaSizeName.NorthAmerica9x11,
            PageMediaSizeName.ISOA2,
            PageMediaSizeName.ISOA3,
            PageMediaSizeName.ISOA3Extra,
            PageMediaSizeName.ISOA4,
            PageMediaSizeName.ISOA4Extra,
            PageMediaSizeName.OtherMetricA4Plus,
            PageMediaSizeName.ISOA5,
            PageMediaSizeName.ISOA5Extra,
            PageMediaSizeName.ISOA6,
            PageMediaSizeName.ISOB4,
            PageMediaSizeName.ISOB4Envelope,
            PageMediaSizeName.ISOB5Envelope,
            PageMediaSizeName.ISOB5Extra,
            PageMediaSizeName.NorthAmericaCSheet,
            PageMediaSizeName.ISOC3Envelope,
            PageMediaSizeName.ISOC4Envelope,
            PageMediaSizeName.ISOC5Envelope,
            PageMediaSizeName.ISOC6Envelope,
            PageMediaSizeName.ISOC6C5Envelope,
            PageMediaSizeName.NorthAmericaDSheet,
            PageMediaSizeName.ISODLEnvelope,
            PageMediaSizeName.NorthAmericaESheet,
            PageMediaSizeName.NorthAmericaNumber10Envelope,
            PageMediaSizeName.NorthAmericaNumber11Envelope,
            PageMediaSizeName.NorthAmericaNumber12Envelope,
            PageMediaSizeName.NorthAmericaNumber14Envelope,
            PageMediaSizeName.NorthAmericaNumber9Envelope,
            PageMediaSizeName.OtherMetricItalianEnvelope,
            PageMediaSizeName.OtherMetricInviteEnvelope,
            PageMediaSizeName.NorthAmericaMonarchEnvelope,
            PageMediaSizeName.NorthAmericaPersonalEnvelope,
            PageMediaSizeName.NorthAmericaExecutive,
            PageMediaSizeName.OtherMetricFolio,
            PageMediaSizeName.NorthAmericaGermanLegalFanfold,
            PageMediaSizeName.NorthAmericaGermanStandardFanfold,
            PageMediaSizeName.JISB4,
            PageMediaSizeName.JISB5,
            PageMediaSizeName.JISB6,
            PageMediaSizeName.JapanChou3Envelope,
            PageMediaSizeName.JapanChou4Envelope,
            PageMediaSizeName.JapanHagakiPostcard,
            PageMediaSizeName.JapanKaku2Envelope,
            PageMediaSizeName.JapanKaku3Envelope,
            PageMediaSizeName.JapanYou4Envelope,
            PageMediaSizeName.JapanDoubleHagakiPostcard,
            PageMediaSizeName.NorthAmericaLegal,
            PageMediaSizeName.NorthAmericaLegalExtra,
            PageMediaSizeName.NorthAmericaLetter,
            PageMediaSizeName.NorthAmericaLetterExtra,
            PageMediaSizeName.NorthAmericaLetterPlus,
            PageMediaSizeName.NorthAmericaNote,
            PageMediaSizeName.PRC1Envelope,
            PageMediaSizeName.PRC10Envelope,
            PageMediaSizeName.PRC3Envelope,
            PageMediaSizeName.PRC4Envelope,
            PageMediaSizeName.PRC5Envelope,
            PageMediaSizeName.PRC6Envelope,
            PageMediaSizeName.PRC7Envelope,
            PageMediaSizeName.PRC8Envelope,
            PageMediaSizeName.PRC9Envelope,
            PageMediaSizeName.NorthAmericaQuarto,
            PageMediaSizeName.NorthAmericaStatement,
            PageMediaSizeName.NorthAmericaSuperA,
            PageMediaSizeName.NorthAmericaSuperB,
            PageMediaSizeName.NorthAmericaTabloid,
        };
    }
}